@@ -296,7 +296,7 @@ public class TestNetworkTopologyImpl {
   }
 
   @Test
-  public void testGetNodesWithLevel() {
+  public void testGetNumOfNodesWithLevel() {
     int maxLevel = cluster.getMaxLevel();
     try {
       assertEquals(1, cluster.getNumOfNodes(0));
@@ -333,6 +333,30 @@ public class TestNetworkTopologyImpl {
     assertEquals(dataNodes.length, cluster.getNumOfNodes(maxLevel));
   }
 
+  @Test
+  public void testGetNodesWithLevel() {
+    int maxLevel = cluster.getMaxLevel();
+    try {
+      assertNotNull(cluster.getNodes(0));
+      fail("level 0 is not supported");
+    } catch (IllegalArgumentException e) {
+      assertTrue(e.getMessage().startsWith("Invalid level"));
+    }
+
+    try {
+      assertNotNull(cluster.getNodes(maxLevel + 1));
+      fail("level out of scope");
+    } catch (IllegalArgumentException e) {
+      assertTrue(e.getMessage().startsWith("Invalid level"));
+    }
+
+    // root node
+    assertEquals(1, cluster.getNodes(1).size());
+    // leaf nodes
+    assertEquals(dataNodes.length, cluster.getNodes(maxLevel).size());
+    assertEquals(dataNodes.length, cluster.getNodes(maxLevel).size());
+  }
+
   @Test
   public void testChooseRandomSimple() {
     String path =
@@ -610,11 +634,12 @@ public class TestNetworkTopologyImpl {
         while (!pathList.get(0).equals(ROOT)) {
           int ancestorGen = cluster.getMaxLevel() - 1;
           while (ancestorGen > 0) {
+            Node affinityNode = dataNodes[k];
             for (Node[] list : excludedNodeLists) {
               List<Node> excludedList = Arrays.asList(list);
               frequency = pickNodes(leafNum, pathList, excludedList,
-                  dataNodes[k], ancestorGen);
-              Node affinityAncestor = dataNodes[k].getAncestor(ancestorGen);
+                  affinityNode, ancestorGen);
+              Node affinityAncestor = affinityNode.getAncestor(ancestorGen);
               for (Node key : dataNodes) {
                 if (affinityAncestor != null) {
                   if (frequency.get(key) > 0) {
@@ -628,6 +653,9 @@ public class TestNetworkTopologyImpl {
                       pathList.stream().anyMatch(path ->
                           key.getNetworkFullPath().startsWith(path))) {
                     continue;
+                  } else if (key.getNetworkFullPath().equals(
+                      affinityNode.getNetworkFullPath())) {
+                    continue;
                   } else {
                     fail("Node is not picked when sequentially going " +
                         "through ancestor node's leaf nodes. node:" +
@@ -852,6 +880,31 @@ public class TestNetworkTopologyImpl {
     }
   }
 
+  @Test
+  public void testSingleNodeRackWithAffinityNode() {
+    // network topology with default cost
+    List<NodeSchema> schemas = new ArrayList<>();
+    schemas.add(ROOT_SCHEMA);
+    schemas.add(RACK_SCHEMA);
+    schemas.add(LEAF_SCHEMA);
+
+    NodeSchemaManager manager = NodeSchemaManager.getInstance();
+    manager.init(schemas.toArray(new NodeSchema[0]), true);
+    NetworkTopology newCluster =
+        new NetworkTopologyImpl(manager);
+    Node node = createDatanode("1.1.1.1", "/r1");
+    newCluster.add(node);
+    Node chosenNode =
+        newCluster.chooseRandom(NetConstants.ROOT, null, null, node, 0);
+    assertNull(chosenNode);
+    chosenNode = newCluster.chooseRandom(NetConstants.ROOT, null,
+        Arrays.asList(node), node, 0);
+    assertNull(chosenNode);
+    chosenNode = newCluster.chooseRandom(NetConstants.ROOT,
+        Arrays.asList(node.getNetworkFullPath()), Arrays.asList(node), node, 0);
+    assertNull(chosenNode);
+  }
+
   private static Node createDatanode(String name, String path) {
     return new NodeImpl(name, path, NetConstants.NODE_COST_DEFAULT);
   }
