@@ -75,6 +75,7 @@ import org.apache.hadoop.util.ReflectionUtils;
 import org.apache.hadoop.util.Shell;
 import org.apache.hadoop.util.StringUtils;
 
+import com.google.common.annotations.VisibleForTesting;
 import com.google.common.base.Preconditions;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Lists;
@@ -151,7 +152,16 @@ public final class HttpServer2 implements FilterContainer {
   // idle timeout in milliseconds
   private static final String HTTP_IDLE_TIMEOUT_MS_KEY =
       "hadoop.http.idle_timeout.ms";
-  private static final int HTTP_IDLE_TIMEOUT_MS_DEFAULT = 10000;
+
+  /**
+   *  This value will never be used because the default value is set in
+   *  core-default.xml from hadoop-common and ozone-default.xml. The value 60k
+   *  here is aligned to the value in ozone-default.xml.
+   *
+   *  TODO: default values for other properties defined here are likely to be
+   *   ignored if defaults are defined in hadoop-common as well.
+   **/
+  private static final int HTTP_IDLE_TIMEOUT_MS_DEFAULT = 60000;
   private static final String HTTP_TEMP_DIR_KEY = "hadoop.http.temp.dir";
 
   private static final String FILTER_INITIALIZER_PROPERTY
@@ -1212,6 +1222,7 @@ public final class HttpServer2 implements FilterContainer {
     } catch (IOException e) {
       throw e;
     } catch (InterruptedException e) {
+      Thread.currentThread().interrupt();
       throw (IOException) new InterruptedIOException(
           "Interrupted while starting HTTP server").initCause(e);
     } catch (Exception e) {
@@ -1752,4 +1763,9 @@ public final class HttpServer2 implements FilterContainer {
         splitVal[1]);
     return headers;
   }
+
+  @VisibleForTesting
+  protected List<ServerConnector> getListeners() {
+    return listeners;
+  }
 }
