@@ -18,7 +18,6 @@
 package org.apache.hadoop.hdds.scm.block;
 
 import java.io.IOException;
-import java.util.Collections;
 import java.util.List;
 import java.util.UUID;
 import java.util.Set;
@@ -27,7 +26,6 @@ import java.util.LinkedHashSet;
 import java.util.ArrayList;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.atomic.AtomicInteger;
-import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
 import java.util.stream.Collectors;
@@ -37,16 +35,19 @@ import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerBlocksDeletionACKProto;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerBlocksDeletionACKProto.DeleteBlockTransactionResult;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.DeletedBlocksTransaction;
+import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.CommandStatus;
 import org.apache.hadoop.hdds.scm.command.CommandStatusReportHandler.DeleteBlockStatus;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
+import org.apache.hadoop.hdds.scm.container.ContainerManager;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.ContainerNotFoundException;
-import org.apache.hadoop.hdds.scm.container.ContainerManager;
 import org.apache.hadoop.hdds.scm.container.ContainerReplica;
-import org.apache.hadoop.hdds.scm.metadata.SCMMetadataStore;
+import org.apache.hadoop.hdds.scm.ha.SCMContext;
+import org.apache.hadoop.hdds.scm.ha.SCMRatisServer;
+import org.apache.hadoop.hdds.scm.ha.SequenceIdGenerator;
+import org.apache.hadoop.hdds.scm.metadata.DBTransactionBuffer;
 import org.apache.hadoop.hdds.server.events.EventHandler;
 import org.apache.hadoop.hdds.server.events.EventPublisher;
-import org.apache.hadoop.hdds.utils.db.BatchOperation;
 import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.hdds.utils.db.TableIterator;
 
@@ -54,6 +55,8 @@ import com.google.common.collect.Lists;
 import static java.lang.Math.min;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_BLOCK_DELETION_MAX_RETRY;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys.OZONE_SCM_BLOCK_DELETION_MAX_RETRY_DEFAULT;
+import static org.apache.hadoop.hdds.scm.ha.SequenceIdGenerator.DEL_TXN_ID;
+
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -72,31 +75,33 @@ public class DeletedBlockLogImpl
 
   public static final Logger LOG =
       LoggerFactory.getLogger(DeletedBlockLogImpl.class);
-  private static final DeletedBlocksTransaction.Builder DUMMY_TXN_BUILDER =
-      DeletedBlocksTransaction.newBuilder().setContainerID(1).setCount(1);
 
   private final int maxRetry;
   private final ContainerManager containerManager;
-  private final SCMMetadataStore scmMetadataStore;
   private final Lock lock;
   // Maps txId to set of DNs which are successful in committing the transaction
   private Map<Long, Set<UUID>> transactionToDNsCommitMap;
   // Maps txId to its retry counts;
-  private Map<Long, Integer> transactionRetryCountMap;
-
-  private final AtomicLong largestTxnId;
-  // largest transactionId is stored at largestTxnIdHolderKey
-  private final long largestTxnIdHolderKey = 0L;
-
+  private Map<Long, Integer> transactionToRetryCountMap;
+  // The access to DeletedBlocksTXTable is protected by
+  // DeletedBlockLogStateManager.
+  private final DeletedBlockLogStateManager deletedBlockLogStateManager;
+  private final SCMContext scmContext;
+  private final SequenceIdGenerator sequenceIdGen;
+  private final ScmBlockDeletingServiceMetrics metrics;
 
+  @SuppressWarnings("parameternumber")
   public DeletedBlockLogImpl(ConfigurationSource conf,
-                             ContainerManager containerManager,
-                             SCMMetadataStore scmMetadataStore)
-      throws IOException {
+      ContainerManager containerManager,
+      SCMRatisServer ratisServer,
+      Table<Long, DeletedBlocksTransaction> deletedBlocksTXTable,
+      DBTransactionBuffer dbTxBuffer,
+      SCMContext scmContext,
+      SequenceIdGenerator sequenceIdGen,
+      ScmBlockDeletingServiceMetrics metrics) {
     maxRetry = conf.getInt(OZONE_SCM_BLOCK_DELETION_MAX_RETRY,
         OZONE_SCM_BLOCK_DELETION_MAX_RETRY_DEFAULT);
     this.containerManager = containerManager;
-    this.scmMetadataStore = scmMetadataStore;
     this.lock = new ReentrantLock();
 
     // transactionToDNsCommitMap is updated only when
@@ -104,45 +109,18 @@ public class DeletedBlockLogImpl
 
     // maps transaction to dns which have committed it.
     transactionToDNsCommitMap = new ConcurrentHashMap<>();
-    transactionRetryCountMap = new ConcurrentHashMap<>();
-    this.largestTxnId = new AtomicLong(this.getLargestRecordedTXID());
-  }
-
-  public Long getNextDeleteBlockTXID() {
-    return this.largestTxnId.incrementAndGet();
-  }
-
-  public Long getCurrentTXID() {
-    return this.largestTxnId.get();
-  }
-
-  /**
-   * Returns the largest recorded TXID from the DB.
-   *
-   * @return Long
-   * @throws IOException
-   */
-  private long getLargestRecordedTXID() throws IOException {
-    DeletedBlocksTransaction txn =
-        scmMetadataStore.getDeletedBlocksTXTable().get(largestTxnIdHolderKey);
-    long txnId = txn != null ? txn.getTxID() : 0L;
-    if (txn == null) {
-      // HDDS-4477 adds largestTxnIdHolderKey to table for storing largest
-      // transactionId. In case the key does not exist, fetch largest
-      // transactionId from existing transactions and update
-      // largestTxnIdHolderKey with same.
-      try (TableIterator<Long,
-              ? extends Table.KeyValue<Long, DeletedBlocksTransaction>> txIter =
-              getIterator()) {
-        txIter.seekToLast();
-        txnId = txIter.key() != null ? txIter.key() : 0L;
-        if (txnId > 0) {
-          scmMetadataStore.getDeletedBlocksTXTable().put(largestTxnIdHolderKey,
-              DUMMY_TXN_BUILDER.setTxID(txnId).build());
-        }
-      }
-    }
-    return txnId;
+    transactionToRetryCountMap = new ConcurrentHashMap<>();
+    this.deletedBlockLogStateManager = DeletedBlockLogStateManagerImpl
+        .newBuilder()
+        .setConfiguration(conf)
+        .setDeletedBlocksTable(deletedBlocksTXTable)
+        .setContainerManager(containerManager)
+        .setRatisServer(ratisServer)
+        .setSCMDBTransactionBuffer(dbTxBuffer)
+        .build();
+    this.scmContext = scmContext;
+    this.sequenceIdGen = sequenceIdGen;
+    this.metrics = metrics;
   }
 
   @Override
@@ -153,7 +131,7 @@ public class DeletedBlockLogImpl
       final List<DeletedBlocksTransaction> failedTXs = Lists.newArrayList();
       try (TableIterator<Long,
           ? extends Table.KeyValue<Long, DeletedBlocksTransaction>> iter =
-               getIterator()) {
+               deletedBlockLogStateManager.getReadOnlyIterator()) {
         while (iter.hasNext()) {
           DeletedBlocksTransaction delTX = iter.next().getValue();
           if (delTX.getCount() == -1) {
@@ -175,58 +153,35 @@ public class DeletedBlockLogImpl
    */
   @Override
   public void incrementCount(List<Long> txIDs) throws IOException {
-    for (Long txID : txIDs) {
-      lock.lock();
-      try {
-        DeletedBlocksTransaction block =
-            scmMetadataStore.getDeletedBlocksTXTable().get(txID);
-        if (block == null) {
-          if (LOG.isDebugEnabled()) {
-            // This can occur due to race condition between retry and old
-            // service task where old task removes the transaction and the new
-            // task is resending
-            LOG.debug("Deleted TXID {} not found.", txID);
-          }
-          continue;
-        }
-
+    lock.lock();
+    try {
+      ArrayList<Long> txIDsToUpdate = new ArrayList<>();
+      for (Long txID : txIDs) {
         int currentCount =
-            transactionRetryCountMap.getOrDefault(txID, block.getCount());
-        if (currentCount > -1) {
-          int nextCount = currentCount + 1;
-          DeletedBlocksTransaction.Builder builder = block.toBuilder();
-          if (nextCount > maxRetry) {
-            // if the retry time exceeds the maxRetry value
-            // then set the retry value to -1, stop retrying, admins can
-            // analyze those blocks and purge them manually by SCMCli.
-            builder.setCount(-1);
-            scmMetadataStore.getDeletedBlocksTXTable().put(txID,
-                builder.build());
-            transactionRetryCountMap.remove(txID);
-          } else if (nextCount % 100 == 0) {
-            // write retry count after every 100 retries into DB.
-            builder.setCount(nextCount);
-            scmMetadataStore.getDeletedBlocksTXTable().put(txID,
-                builder.build());
-            transactionRetryCountMap.put(txID, nextCount);
-          } else {
-            transactionRetryCountMap.put(txID, nextCount);
+            transactionToRetryCountMap.getOrDefault(txID, 0);
+        if (currentCount > maxRetry) {
+          continue;
+        } else {
+          currentCount += 1;
+          if (currentCount > maxRetry) {
+            txIDsToUpdate.add(txID);
           }
+          transactionToRetryCountMap.put(txID, currentCount);
         }
-      } catch (IOException ex) {
-        LOG.warn("Cannot increase count for txID " + txID, ex);
-        // We do not throw error here, since we don't want to abort the loop.
-        // Just log and continue processing the rest of txids.
-      } finally {
-        lock.unlock();
       }
+
+      if (!txIDsToUpdate.isEmpty()) {
+        deletedBlockLogStateManager
+            .increaseRetryCountOfTransactionInDB(txIDsToUpdate);
+      }
+    } finally {
+      lock.unlock();
     }
   }
 
 
-  private DeletedBlocksTransaction constructNewTransaction(long txID,
-                                                           long containerID,
-                                                           List<Long> blocks) {
+  private DeletedBlocksTransaction constructNewTransaction(
+      long txID, long containerID, List<Long> blocks) {
     return DeletedBlocksTransaction.newBuilder()
         .setTxID(txID)
         .setContainerID(containerID)
@@ -248,17 +203,20 @@ public class DeletedBlockLogImpl
       List<DeleteBlockTransactionResult> transactionResults, UUID dnID) {
     lock.lock();
     try {
+      ArrayList<Long> txIDsToBeDeleted = new ArrayList<>();
       Set<UUID> dnsWithCommittedTxn;
       for (DeleteBlockTransactionResult transactionResult :
           transactionResults) {
         if (isTransactionFailed(transactionResult)) {
+          metrics.incrBlockDeletionTransactionFailure();
           continue;
         }
         try {
+          metrics.incrBlockDeletionTransactionSuccess();
           long txID = transactionResult.getTxID();
           // set of dns which have successfully committed transaction txId.
           dnsWithCommittedTxn = transactionToDNsCommitMap.get(txID);
-          final ContainerID containerId = ContainerID.valueof(
+          final ContainerID containerId = ContainerID.valueOf(
               transactionResult.getContainerID());
           if (dnsWithCommittedTxn == null) {
             // Mostly likely it's a retried delete command response.
@@ -280,18 +238,18 @@ public class DeletedBlockLogImpl
           // corresponding nodes commit the txn. It is required to check that
           // the nodes returned in the pipeline match the replication factor.
           if (min(replicas.size(), dnsWithCommittedTxn.size())
-              >= container.getReplicationFactor().getNumber()) {
+              >= container.getReplicationConfig().getRequiredNodes()) {
             List<UUID> containerDns = replicas.stream()
                 .map(ContainerReplica::getDatanodeDetails)
                 .map(DatanodeDetails::getUuid)
                 .collect(Collectors.toList());
             if (dnsWithCommittedTxn.containsAll(containerDns)) {
               transactionToDNsCommitMap.remove(txID);
-              transactionRetryCountMap.remove(txID);
+              transactionToRetryCountMap.remove(txID);
               if (LOG.isDebugEnabled()) {
                 LOG.debug("Purging txId={} from block deletion log", txID);
               }
-              scmMetadataStore.getDeletedBlocksTXTable().delete(txID);
+              txIDsToBeDeleted.add(txID);
             }
           }
           if (LOG.isDebugEnabled()) {
@@ -303,6 +261,13 @@ public class DeletedBlockLogImpl
               transactionResult.getTxID(), e);
         }
       }
+      try {
+        deletedBlockLogStateManager.removeTransactionsFromDB(txIDsToBeDeleted);
+        metrics.incrBlockDeletionTransactionCompleted(txIDsToBeDeleted.size());
+      } catch (IOException e) {
+        LOG.warn("Could not commit delete block transactions: "
+            + txIDsToBeDeleted, e);
+      }
     } finally {
       lock.unlock();
     }
@@ -322,20 +287,6 @@ public class DeletedBlockLogImpl
     return false;
   }
 
-  /**
-   * {@inheritDoc}
-   *
-   * @param containerID - container ID.
-   * @param blocks      - blocks that belong to the same container.
-   * @throws IOException
-   */
-  @Override
-  public void addTransaction(long containerID, List<Long> blocks)
-      throws IOException {
-    Map<Long, List<Long>> map = Collections.singletonMap(containerID, blocks);
-    addTransactions(map);
-  }
-
   @Override
   public int getNumOfValidTransactions() throws IOException {
     lock.lock();
@@ -343,7 +294,7 @@ public class DeletedBlockLogImpl
       final AtomicInteger num = new AtomicInteger(0);
       try (TableIterator<Long,
           ? extends Table.KeyValue<Long, DeletedBlocksTransaction>> iter =
-               getIterator()) {
+               deletedBlockLogStateManager.getReadOnlyIterator()) {
         while (iter.hasNext()) {
           DeletedBlocksTransaction delTX = iter.next().getValue();
           if (delTX.getCount() > -1) {
@@ -357,6 +308,32 @@ public class DeletedBlockLogImpl
     }
   }
 
+  @Override
+  public void reinitialize(
+      Table<Long, DeletedBlocksTransaction> deletedTable) {
+    // we don't need handle transactionToDNsCommitMap and
+    // deletedBlockLogStateManager, since they will be cleared
+    // when becoming leader.
+    deletedBlockLogStateManager.reinitialize(deletedTable);
+  }
+
+  /**
+   * Called in SCMStateMachine#notifyLeaderChanged when current SCM becomes
+   *  leader.
+   */
+  public void onBecomeLeader() {
+    transactionToDNsCommitMap.clear();
+    transactionToRetryCountMap.clear();
+  }
+
+  /**
+   * Called in SCMDBTransactionBuffer#flush when the cached deleting operations
+   * are flushed.
+   */
+  public void onFlush() {
+    deletedBlockLogStateManager.onFlush();
+  }
+
   /**
    * {@inheritDoc}
    *
@@ -367,20 +344,18 @@ public class DeletedBlockLogImpl
   public void addTransactions(Map<Long, List<Long>> containerBlocksMap)
       throws IOException {
     lock.lock();
-    try (BatchOperation batch = scmMetadataStore.getStore()
-        .initBatchOperation()) {
-      for (Map.Entry<Long, List<Long>> entry : containerBlocksMap.entrySet()) {
-        long nextTXID = getNextDeleteBlockTXID();
-        scmMetadataStore.getDeletedBlocksTXTable().putWithBatch(batch, nextTXID,
-            constructNewTransaction(nextTXID, entry.getKey(),
-                entry.getValue()));
+    try {
+      ArrayList<DeletedBlocksTransaction> txsToBeAdded = new ArrayList<>();
+      for (Map.Entry< Long, List< Long > > entry :
+          containerBlocksMap.entrySet()) {
+        long nextTXID = sequenceIdGen.getNextId(DEL_TXN_ID);
+        DeletedBlocksTransaction tx = constructNewTransaction(nextTXID,
+            entry.getKey(), entry.getValue());
+        txsToBeAdded.add(tx);
       }
-      // Add a dummy transaction to store the largestTransactionId at
-      // largestTxnIdHolderKey
-      scmMetadataStore.getDeletedBlocksTXTable()
-          .putWithBatch(batch, largestTxnIdHolderKey,
-              DUMMY_TXN_BUILDER.setTxID(getCurrentTXID()).build());
-      scmMetadataStore.getStore().commitBatchOperation(batch);
+
+      deletedBlockLogStateManager.addTransactionsToDB(txsToBeAdded);
+      metrics.incrBlockDeletionTransactionCreated(txsToBeAdded.size());
     } finally {
       lock.unlock();
     }
@@ -394,7 +369,7 @@ public class DeletedBlockLogImpl
       DatanodeDeletedBlockTransactions transactions) {
     try {
       Set<ContainerReplica> replicas = containerManager
-          .getContainerReplicas(ContainerID.valueof(tx.getContainerID()));
+          .getContainerReplicas(ContainerID.valueOf(tx.getContainerID()));
       for (ContainerReplica replica : replicas) {
         UUID dnID = replica.getDatanodeDetails().getUuid();
         Set<UUID> dnsWithTransactionCommitted =
@@ -420,14 +395,13 @@ public class DeletedBlockLogImpl
           new DatanodeDeletedBlockTransactions();
       try (TableIterator<Long,
           ? extends Table.KeyValue<Long, DeletedBlocksTransaction>> iter =
-               getIterator()) {
+               deletedBlockLogStateManager.getReadOnlyIterator()) {
         int numBlocksAdded = 0;
-        List<DeletedBlocksTransaction> txnsToBePurged =
-            new ArrayList<>();
+        ArrayList<Long> txIDs = new ArrayList<>();
         while (iter.hasNext() && numBlocksAdded < blockDeletionLimit) {
           Table.KeyValue<Long, DeletedBlocksTransaction> keyValue = iter.next();
           DeletedBlocksTransaction txn = keyValue.getValue();
-          final ContainerID id = ContainerID.valueof(txn.getContainerID());
+          final ContainerID id = ContainerID.valueOf(txn.getContainerID());
           try {
             if (txn.getCount() > -1 && txn.getCount() <= maxRetry
                 && !containerManager.getContainer(id).isOpen()) {
@@ -439,13 +413,12 @@ public class DeletedBlockLogImpl
           } catch (ContainerNotFoundException ex) {
             LOG.warn("Container: " + id + " was not found for the transaction: "
                 + txn);
-            txnsToBePurged.add(txn);
+            txIDs.add(txn.getTxID());
           }
         }
-        purgeTransactions(txnsToBePurged);
-        for (DeletedBlocksTransaction trx : txnsToBePurged) {
-          transactionRetryCountMap.remove(trx.getTxID());
-          transactionToDNsCommitMap.remove(trx.getTxID());
+        if (!txIDs.isEmpty()) {
+          deletedBlockLogStateManager.removeTransactionsFromDB(txIDs);
+          metrics.incrBlockDeletionTransactionCompleted(txIDs.size());
         }
       }
       return transactions;
@@ -454,34 +427,26 @@ public class DeletedBlockLogImpl
     }
   }
 
-  public void purgeTransactions(List<DeletedBlocksTransaction> txnsToBePurged)
-      throws IOException {
-    try (BatchOperation batch = scmMetadataStore.getBatchHandler()
-        .initBatchOperation()) {
-      for (int i = 0; i < txnsToBePurged.size(); i++) {
-        scmMetadataStore.getDeletedBlocksTXTable()
-            .deleteWithBatch(batch, txnsToBePurged.get(i).getTxID());
-      }
-      scmMetadataStore.getBatchHandler().commitBatchOperation(batch);
+  @Override
+  public void onMessage(
+      DeleteBlockStatus deleteBlockStatus, EventPublisher publisher) {
+    if (!scmContext.isLeader()) {
+      LOG.warn("Skip commit transactions since current SCM is not leader.");
+      return;
+    }
+
+    CommandStatus.Status status = deleteBlockStatus.getCmdStatus().getStatus();
+    if (status == CommandStatus.Status.EXECUTED) {
+      ContainerBlocksDeletionACKProto ackProto =
+          deleteBlockStatus.getCmdStatus().getBlockDeletionAck();
+      commitTransactions(ackProto.getResultsList(),
+          UUID.fromString(ackProto.getDnId()));
+      metrics.incrBlockDeletionCommandSuccess();
+    } else if (status == CommandStatus.Status.FAILED){
+      metrics.incrBlockDeletionCommandFailure();
+    } else {
+      LOG.error("Delete Block Command is not executed yet.");
+      return;
     }
   }
-
-  TableIterator<Long,
-      ? extends Table.KeyValue<Long, DeletedBlocksTransaction>> getIterator()
-      throws IOException {
-    TableIterator<Long,
-        ? extends Table.KeyValue<Long, DeletedBlocksTransaction>>
-        iter = scmMetadataStore.getDeletedBlocksTXTable().iterator();
-    iter.seek(largestTxnIdHolderKey + 1);
-    return iter;
-  }
-
-  @Override
-  public void onMessage(DeleteBlockStatus deleteBlockStatus,
-                        EventPublisher publisher) {
-    ContainerBlocksDeletionACKProto ackProto =
-        deleteBlockStatus.getCmdStatus().getBlockDeletionAck();
-    commitTransactions(ackProto.getResultsList(),
-        UUID.fromString(ackProto.getDnId()));
-  }
 }
