@@ -26,13 +26,14 @@ import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.atomic.AtomicLong;
 
+import org.apache.hadoop.hdds.client.RatisReplicationConfig;
+import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.StorageUnit;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleEvent;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.LifeCycleState;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor;
-import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType;
 import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.container.states.ContainerState;
 import org.apache.hadoop.hdds.scm.container.states.ContainerStateMap;
@@ -237,39 +238,33 @@ public class ContainerStateManager {
 
   /**
    * Allocates a new container based on the type, replication etc.
-   *
-   * @param pipelineManager -- Pipeline Manager class.
-   * @param type -- Replication type.
-   * @param replicationFactor - Replication replicationFactor.
-   * @return ContainerWithPipeline
-   * @throws IOException  on Failure.
    */
   ContainerInfo allocateContainer(final PipelineManager pipelineManager,
-      final HddsProtos.ReplicationType type,
-      final HddsProtos.ReplicationFactor replicationFactor, final String owner)
+      final ReplicationConfig replicationConfig, final String owner)
       throws IOException {
     final List<Pipeline> pipelines = pipelineManager
-        .getPipelines(type, replicationFactor, Pipeline.PipelineState.OPEN);
+        .getPipelines(replicationConfig, Pipeline.PipelineState.OPEN);
     Pipeline pipeline;
 
-    boolean bgCreateOne = (type == ReplicationType.RATIS) && replicationFactor
-        == ReplicationFactor.ONE && autoCreateRatisOne;
-    boolean bgCreateThree = (type == ReplicationType.RATIS) && replicationFactor
-        == ReplicationFactor.THREE;
+    boolean bgCreateOne = RatisReplicationConfig
+        .hasFactor(replicationConfig, ReplicationFactor.ONE)
+        && autoCreateRatisOne;
+    boolean bgCreateThree = RatisReplicationConfig
+        .hasFactor(replicationConfig, ReplicationFactor.THREE);
 
     if (!pipelines.isEmpty() && (bgCreateOne || bgCreateThree)) {
       // let background create Ratis pipelines.
       pipeline = pipelines.get((int) containerCount.get() % pipelines.size());
     } else {
       try {
-        pipeline = pipelineManager.createPipeline(type, replicationFactor);
+        pipeline = pipelineManager.createPipeline(replicationConfig);
         pipelineManager.waitPipelineReady(pipeline.getId(), 0);
       } catch (IOException e) {
 
         if (pipelines.isEmpty()) {
           throw new IOException("Could not allocate container. Cannot get any" +
-              " matching pipeline for Type:" + type +
-              ", Factor:" + replicationFactor + ", State:PipelineState.OPEN");
+              " matching pipeline for replicationConfig:" + replicationConfig +
+              ", State:PipelineState.OPEN");
         }
         pipeline = pipelines.get((int) containerCount.get() % pipelines.size());
       }
@@ -309,8 +304,7 @@ public class ContainerStateManager {
         .setOwner(owner)
         .setContainerID(containerID)
         .setDeleteTransactionId(0)
-        .setReplicationFactor(pipeline.getFactor())
-        .setReplicationType(pipeline.getType())
+        .setReplicationConfig(pipeline.getReplicationConfig())
         .build();
     addContainerInfo(containerID, containerInfo, pipelineManager, pipeline);
     if (LOG.isTraceEnabled()) {
@@ -329,7 +323,7 @@ public class ContainerStateManager {
       // In Recon, while adding a 'new' CLOSED container, pipeline will be a
       // random ID, and hence be passed down as null.
       pipelineManager.addContainerToPipeline(pipeline.getId(),
-          ContainerID.valueof(containerID));
+          ContainerID.valueOf(containerID));
     }
     containerStateCount.incrementAndGet(containerInfo.getState());
   }
@@ -371,12 +365,8 @@ public class ContainerStateManager {
   void updateDeleteTransactionId(
       final Map<Long, Long> deleteTransactionMap) {
     deleteTransactionMap.forEach((k, v) -> {
-      try {
-        containers.getContainerInfo(ContainerID.valueof(k))
-            .updateDeleteTransactionId(v);
-      } catch (ContainerNotFoundException e) {
-        LOG.warn("Exception while updating delete transaction id.", e);
-      }
+      containers.getContainerInfo(ContainerID.valueOf(k))
+          .updateDeleteTransactionId(v);
     });
   }
 
@@ -432,18 +422,13 @@ public class ContainerStateManager {
   private ContainerInfo findContainerWithSpace(final long size,
       final NavigableSet<ContainerID> searchSet, final String owner,
       final PipelineID pipelineID) {
-    try {
-      // Get the container with space to meet our request.
-      for (ContainerID id : searchSet) {
-        final ContainerInfo containerInfo = containers.getContainerInfo(id);
-        if (containerInfo.getUsedBytes() + size <= this.containerSize) {
-          containerInfo.updateLastUsedTime();
-          return containerInfo;
-        }
+    // Get the container with space to meet our request.
+    for (ContainerID id : searchSet) {
+      final ContainerInfo containerInfo = containers.getContainerInfo(id);
+      if (containerInfo.getUsedBytes() + size <= this.containerSize) {
+        containerInfo.updateLastUsedTime();
+        return containerInfo;
       }
-    } catch (ContainerNotFoundException e) {
-      // This should not happen!
-      LOG.warn("Exception while finding container with space", e);
     }
     return null;
   }
@@ -476,16 +461,13 @@ public class ContainerStateManager {
    * Returns a set of ContainerIDs that match the Container.
    *
    * @param owner  Owner of the Containers.
-   * @param type - Replication Type of the containers
-   * @param factor - Replication factor of the containers.
+   * @param repConfig - Replication Config of the containers
    * @param state - Current State, like Open, Close etc.
    * @return Set of containers that match the specific query parameters.
    */
   NavigableSet<ContainerID> getMatchingContainerIDs(final String owner,
-      final ReplicationType type, final ReplicationFactor factor,
-      final LifeCycleState state) {
-    return containers.getMatchingContainerIDs(state, owner,
-        factor, type);
+      final ReplicationConfig repConfig, final LifeCycleState state) {
+    return containers.getMatchingContainerIDs(state, owner, repConfig);
   }
 
   /**
@@ -496,7 +478,11 @@ public class ContainerStateManager {
    */
   ContainerInfo getContainer(final ContainerID containerID)
       throws ContainerNotFoundException {
-    return containers.getContainerInfo(containerID);
+    final ContainerInfo container = containers.getContainerInfo(containerID);
+    if (container != null) {
+      return container;
+    }
+    throw new ContainerNotFoundException(containerID.toString());
   }
 
   void close() throws IOException {
@@ -540,6 +526,9 @@ public class ContainerStateManager {
 
   void removeContainer(final ContainerID containerID)
       throws ContainerNotFoundException {
+    if (containers.getContainerInfo(containerID) == null) {
+      throw new ContainerNotFoundException(containerID.toString());
+    }
     containers.removeContainer(containerID);
   }
 
