@@ -19,22 +19,25 @@ package org.apache.hadoop.hdds.scm.block;
 
 import org.apache.commons.io.FileUtils;
 import org.apache.hadoop.hdds.HddsConfigKeys;
+import org.apache.hadoop.hdds.client.RatisReplicationConfig;
+import org.apache.hadoop.hdds.client.StandaloneReplicationConfig;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto
     .StorageContainerDatanodeProtocolProtos.ContainerReplicaProto;
+import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.TestUtils;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
-import org.apache.hadoop.hdds.scm.container.ContainerReplica;
-import org.apache.hadoop.hdds.scm.container.SCMContainerManager;
 import org.apache.hadoop.hdds.scm.container.ContainerManager;
+import org.apache.hadoop.hdds.scm.container.ContainerReplica;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
+import org.apache.hadoop.hdds.scm.ha.SCMHADBTransactionBuffer;
+import org.apache.hadoop.hdds.scm.ha.MockSCMHADBTransactionBuffer;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
 import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor;
-import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType;
 import org.apache.hadoop.hdds.protocol.proto
     .StorageContainerDatanodeProtocolProtos.DeletedBlocksTransaction;
 import org.apache.hadoop.hdds.protocol.proto
@@ -42,7 +45,7 @@ import org.apache.hadoop.hdds.protocol.proto
     .DeleteBlockTransactionResult;
 import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.hdds.utils.db.TableIterator;
-import org.apache.hadoop.test.GenericTestUtils;
+import org.apache.ozone.test.GenericTestUtils;
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
@@ -65,9 +68,11 @@ import java.util.UUID;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.stream.Collectors;
 
+import static org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor.THREE;
 import static org.apache.hadoop.hdds.scm.ScmConfigKeys
     .OZONE_SCM_BLOCK_DELETION_MAX_RETRY;
 import static org.mockito.Matchers.anyObject;
+import static org.mockito.Mockito.doAnswer;
 import static org.mockito.Mockito.when;
 
 /**
@@ -75,25 +80,41 @@ import static org.mockito.Mockito.when;
  */
 public class TestDeletedBlockLog {
 
-  private DeletedBlockLogImpl deletedBlockLog;
+  private  DeletedBlockLogImpl deletedBlockLog;
   private static final int BLOCKS_PER_TXN = 5;
   private OzoneConfiguration conf;
   private File testDir;
   private ContainerManager containerManager;
+  private Table<ContainerID, ContainerInfo> containerTable;
   private StorageContainerManager scm;
   private List<DatanodeDetails> dnList;
+  private SCMHADBTransactionBuffer scmHADBTransactionBuffer;
+  private Map<Long, ContainerInfo> containers = new HashMap<>();
+  private Map<Long, Set<ContainerReplica>> replicas = new HashMap<>();
+  private ScmBlockDeletingServiceMetrics metrics;
 
   @Before
   public void setup() throws Exception {
     testDir = GenericTestUtils.getTestDir(
         TestDeletedBlockLog.class.getSimpleName());
     conf = new OzoneConfiguration();
+    conf.setBoolean(ScmConfigKeys.OZONE_SCM_HA_ENABLE_KEY, true);
     conf.setInt(OZONE_SCM_BLOCK_DELETION_MAX_RETRY, 20);
     conf.set(HddsConfigKeys.OZONE_METADATA_DIRS, testDir.getAbsolutePath());
     scm = TestUtils.getScm(conf);
-    containerManager = Mockito.mock(SCMContainerManager.class);
-    deletedBlockLog = new DeletedBlockLogImpl(conf, containerManager,
-        scm.getScmMetadataStore());
+    containerManager = Mockito.mock(ContainerManager.class);
+    containerTable = scm.getScmMetadataStore().getContainerTable();
+    scmHADBTransactionBuffer =
+        new MockSCMHADBTransactionBuffer(scm.getScmMetadataStore().getStore());
+    metrics = Mockito.mock(ScmBlockDeletingServiceMetrics.class);
+    deletedBlockLog = new DeletedBlockLogImpl(conf,
+        containerManager,
+        scm.getScmHAManager().getRatisServer(),
+        scm.getScmMetadataStore().getDeletedBlocksTXTable(),
+        scmHADBTransactionBuffer,
+        scm.getScmContext(),
+        scm.getSequenceIdGen(),
+        metrics);
     dnList = new ArrayList<>(3);
     setupContainerManager();
   }
@@ -109,10 +130,44 @@ public class TestDeletedBlockLog {
         DatanodeDetails.newBuilder().setUuid(UUID.randomUUID())
             .build());
 
+    when(containerManager.getContainerReplicas(anyObject()))
+        .thenAnswer(invocationOnMock -> {
+          ContainerID cid = (ContainerID) invocationOnMock.getArguments()[0];
+          return replicas.get(cid.getId());
+        });
+    when(containerManager.getContainer(anyObject()))
+        .thenAnswer(invocationOnMock -> {
+          ContainerID cid = (ContainerID) invocationOnMock.getArguments()[0];
+          return containerTable.get(cid);
+        });
+    when(containerManager.getContainers())
+        .thenReturn(new ArrayList<>(containers.values()));
+    doAnswer(invocationOnMock -> {
+      Map<ContainerID, Long> map =
+          (Map<ContainerID, Long>) invocationOnMock.getArguments()[0];
+      for (Map.Entry<ContainerID, Long> e : map.entrySet()) {
+        ContainerInfo info = containers.get(e.getKey().getId());
+        try {
+          Assert.assertTrue(e.getValue() > info.getDeleteTransactionId());
+        } catch (AssertionError err) {
+          throw new Exception("New TxnId " + e.getValue() + " < " + info
+              .getDeleteTransactionId());
+        }
+        info.updateDeleteTransactionId(e.getValue());
+        scmHADBTransactionBuffer.addToBuffer(containerTable, e.getKey(), info);
+      }
+      return null;
+    }).when(containerManager).updateDeleteTransactionId(anyObject());
+  }
+
+  private void updateContainerMetadata(long cid) throws IOException {
     final ContainerInfo container =
-        new ContainerInfo.Builder().setContainerID(1)
-            .setReplicationFactor(ReplicationFactor.THREE)
+        new ContainerInfo.Builder()
+            .setContainerID(cid)
+            .setReplicationConfig(new RatisReplicationConfig(THREE))
             .setState(HddsProtos.LifeCycleState.CLOSED)
+            .setOwner("TestDeletedBlockLog")
+            .setPipelineID(PipelineID.randomId())
             .build();
     final Set<ContainerReplica> replicaSet = dnList.stream()
         .map(datanodeDetails -> ContainerReplica.newBuilder()
@@ -121,11 +176,9 @@ public class TestDeletedBlockLog {
             .setDatanodeDetails(datanodeDetails)
             .build())
         .collect(Collectors.toSet());
-
-    when(containerManager.getContainerReplicas(anyObject()))
-        .thenReturn(replicaSet);
-    when(containerManager.getContainer(anyObject()))
-        .thenReturn(container);
+    containers.put(cid, container);
+    containerTable.put(ContainerID.valueOf(cid), container);
+    replicas.put(cid, replicaSet);
   }
 
   @After
@@ -136,13 +189,14 @@ public class TestDeletedBlockLog {
     FileUtils.deleteDirectory(testDir);
   }
 
-  private Map<Long, List<Long>> generateData(int dataSize) {
+  private Map<Long, List<Long>> generateData(int dataSize) throws IOException {
     Map<Long, List<Long>> blockMap = new HashMap<>();
     Random random = new Random(1);
     int continerIDBase = random.nextInt(100);
     int localIDBase = random.nextInt(1000);
     for (int i = 0; i < dataSize; i++) {
       long containerID = continerIDBase + i;
+      updateContainerMetadata(containerID);
       List<Long> blocks = new ArrayList<>();
       for (int j = 0; j < BLOCKS_PER_TXN; j++)  {
         long localID = localIDBase + j;
@@ -153,31 +207,48 @@ public class TestDeletedBlockLog {
     return blockMap;
   }
 
+  private void addTransactions(Map<Long, List<Long>> containerBlocksMap,
+      boolean shouldFlush)
+      throws IOException {
+    deletedBlockLog.addTransactions(containerBlocksMap);
+    if (shouldFlush) {
+      scmHADBTransactionBuffer.flush();
+    }
+  }
+
+  private void incrementCount(List<Long> txIDs) throws IOException {
+    deletedBlockLog.incrementCount(txIDs);
+    scmHADBTransactionBuffer.flush();
+  }
+
   private void commitTransactions(
       List<DeleteBlockTransactionResult> transactionResults,
-      DatanodeDetails... dns) {
+      DatanodeDetails... dns) throws IOException {
     for (DatanodeDetails dnDetails : dns) {
       deletedBlockLog
           .commitTransactions(transactionResults, dnDetails.getUuid());
     }
+    scmHADBTransactionBuffer.flush();
   }
 
   private void commitTransactions(
-      List<DeleteBlockTransactionResult> transactionResults) {
+      List<DeleteBlockTransactionResult> transactionResults)
+      throws IOException {
     commitTransactions(transactionResults,
         dnList.toArray(new DatanodeDetails[3]));
   }
 
   private void commitTransactions(
       Collection<DeletedBlocksTransaction> deletedBlocksTransactions,
-      DatanodeDetails... dns) {
+      DatanodeDetails... dns) throws IOException {
     commitTransactions(deletedBlocksTransactions.stream()
         .map(this::createDeleteBlockTransactionResult)
         .collect(Collectors.toList()), dns);
   }
 
   private void commitTransactions(
-      Collection<DeletedBlocksTransaction> deletedBlocksTransactions) {
+      Collection<DeletedBlocksTransaction> deletedBlocksTransactions)
+      throws IOException {
     commitTransactions(deletedBlocksTransactions.stream()
         .map(this::createDeleteBlockTransactionResult)
         .collect(Collectors.toList()));
@@ -203,14 +274,37 @@ public class TestDeletedBlockLog {
     return txns.stream().distinct().collect(Collectors.toList());
   }
 
+  @Test
+  public void testContainerManagerTransactionId() throws Exception {
+    // Initially all containers should have deleteTransactionId as 0
+    for (ContainerInfo containerInfo : containerManager.getContainers()) {
+      Assert.assertEquals(0, containerInfo.getDeleteTransactionId());
+    }
+
+    // Create 30 TXs
+    addTransactions(generateData(30), false);
+    // Since transactions are not yet flushed deleteTransactionId should be
+    // 0 for all containers
+    Assert.assertEquals(0, getTransactions(1000).size());
+    for (ContainerInfo containerInfo : containerManager.getContainers()) {
+      Assert.assertEquals(0, containerInfo.getDeleteTransactionId());
+    }
+
+    scmHADBTransactionBuffer.flush();
+    // After flush there should be 30 transactions in deleteTable
+    // All containers should have positive deleteTransactionId
+    Assert.assertEquals(30, getTransactions(1000).size());
+    for (ContainerInfo containerInfo : containerManager.getContainers()) {
+      Assert.assertTrue(containerInfo.getDeleteTransactionId() > 0);
+    }
+  }
+
   @Test
   public void testIncrementCount() throws Exception {
     int maxRetry = conf.getInt(OZONE_SCM_BLOCK_DELETION_MAX_RETRY, 20);
 
     // Create 30 TXs in the log.
-    for (Map.Entry<Long, List<Long>> entry : generateData(30).entrySet()){
-      deletedBlockLog.addTransaction(entry.getKey(), entry.getValue());
-    }
+    addTransactions(generateData(30), true);
 
     // This will return all TXs, total num 30.
     List<DeletedBlocksTransaction> blocks =
@@ -219,12 +313,12 @@ public class TestDeletedBlockLog {
         .collect(Collectors.toList());
 
     for (int i = 0; i < maxRetry; i++) {
-      deletedBlockLog.incrementCount(txIDs);
+      incrementCount(txIDs);
     }
 
     // Increment another time so it exceed the maxRetry.
     // On this call, count will be set to -1 which means TX eventually fails.
-    deletedBlockLog.incrementCount(txIDs);
+    incrementCount(txIDs);
     blocks = getTransactions(40 * BLOCKS_PER_TXN);
     for (DeletedBlocksTransaction block : blocks) {
       Assert.assertEquals(-1, block.getCount());
@@ -235,58 +329,9 @@ public class TestDeletedBlockLog {
     Assert.assertEquals(blocks.size(), 0);
   }
 
-  @Test
-  public void testIncrementCountLessFrequentWritingToDB() throws Exception {
-    OzoneConfiguration testConf = OzoneConfiguration.of(conf);
-    testConf.setInt(OZONE_SCM_BLOCK_DELETION_MAX_RETRY, 120);
-
-    deletedBlockLog = new DeletedBlockLogImpl(testConf, containerManager,
-        scm.getScmMetadataStore());
-
-    for (Map.Entry<Long, List<Long>> entry :
-        generateData(1).entrySet()) {
-      deletedBlockLog.addTransaction(entry.getKey(), entry.getValue());
-    }
-
-    List<DeletedBlocksTransaction> blocks =
-        getTransactions(40 * BLOCKS_PER_TXN);
-    List<Long> txIDs = blocks.stream().map(DeletedBlocksTransaction::getTxID)
-        .collect(Collectors.toList());
-
-    for (int i = 0; i < 50; i++) {
-      deletedBlockLog.incrementCount(txIDs);
-    }
-    blocks = getTransactions(40 * BLOCKS_PER_TXN);
-    for (DeletedBlocksTransaction block : blocks) {
-      // block count should not be updated as there are only 50 retries.
-      Assert.assertEquals(0, block.getCount());
-    }
-
-    for (int i = 0; i < 60; i++) {
-      deletedBlockLog.incrementCount(txIDs);
-    }
-    blocks = getTransactions(40 * BLOCKS_PER_TXN);
-    for (DeletedBlocksTransaction block : blocks) {
-      // block count should be updated to 100 as there are already 110 retries.
-      Assert.assertEquals(100, block.getCount());
-    }
-
-    for (int i = 0; i < 50; i++) {
-      deletedBlockLog.incrementCount(txIDs);
-    }
-    blocks = getTransactions(40 * BLOCKS_PER_TXN);
-    for (DeletedBlocksTransaction block : blocks) {
-      // block count should be updated to -1 as retry count exceeds maxRetry
-      // (i.e. 160 > maxRetry which is 120).
-      Assert.assertEquals(-1, block.getCount());
-    }
-  }
-
   @Test
   public void testCommitTransactions() throws Exception {
-    for (Map.Entry<Long, List<Long>> entry : generateData(50).entrySet()){
-      deletedBlockLog.addTransaction(entry.getKey(), entry.getValue());
-    }
+    addTransactions(generateData(50), true);
     List<DeletedBlocksTransaction> blocks =
         getTransactions(20 * BLOCKS_PER_TXN);
     // Add an invalid txn.
@@ -320,10 +365,7 @@ public class TestDeletedBlockLog {
     for (int i = 0; i < 100; i++) {
       int state = random.nextInt(4);
       if (state == 0) {
-        for (Map.Entry<Long, List<Long>> entry :
-            generateData(10).entrySet()){
-          deletedBlockLog.addTransaction(entry.getKey(), entry.getValue());
-        }
+        addTransactions(generateData(10), true);
         added += 10;
       } else if (state == 1) {
         blocks = getTransactions(20);
@@ -331,7 +373,7 @@ public class TestDeletedBlockLog {
         for (DeletedBlocksTransaction block : blocks) {
           txIDs.add(block.getTxID());
         }
-        deletedBlockLog.incrementCount(txIDs);
+        incrementCount(txIDs);
       } else if (state == 2) {
         commitTransactions(blocks);
         committed += blocks.size();
@@ -340,7 +382,7 @@ public class TestDeletedBlockLog {
         // verify the number of added and committed.
         try (TableIterator<Long,
             ? extends Table.KeyValue<Long, DeletedBlocksTransaction>> iter =
-            deletedBlockLog.getIterator()) {
+            scm.getScmMetadataStore().getDeletedBlocksTXTable().iterator()) {
           AtomicInteger count = new AtomicInteger();
           iter.forEachRemaining((keyValue) -> count.incrementAndGet());
           Assert.assertEquals(added, count.get() + committed);
@@ -353,14 +395,18 @@ public class TestDeletedBlockLog {
 
   @Test
   public void testPersistence() throws Exception {
-    for (Map.Entry<Long, List<Long>> entry : generateData(50).entrySet()){
-      deletedBlockLog.addTransaction(entry.getKey(), entry.getValue());
-    }
+    addTransactions(generateData(50), true);
     // close db and reopen it again to make sure
     // transactions are stored persistently.
     deletedBlockLog.close();
-    deletedBlockLog = new DeletedBlockLogImpl(conf, containerManager,
-        scm.getScmMetadataStore());
+    deletedBlockLog = new DeletedBlockLogImpl(conf,
+        containerManager,
+        scm.getScmHAManager().getRatisServer(),
+        scm.getScmMetadataStore().getDeletedBlocksTXTable(),
+        scmHADBTransactionBuffer,
+        scm.getScmContext(),
+        scm.getSequenceIdGen(),
+        metrics);
     List<DeletedBlocksTransaction> blocks =
         getTransactions(BLOCKS_PER_TXN * 10);
     Assert.assertEquals(10, blocks.size());
@@ -372,11 +418,17 @@ public class TestDeletedBlockLog {
     // close db and reopen it again to make sure
     // currentTxnID = 50
     deletedBlockLog.close();
-    deletedBlockLog = new DeletedBlockLogImpl(conf, containerManager,
-        scm.getScmMetadataStore());
+    new DeletedBlockLogImpl(conf,
+        containerManager,
+        scm.getScmHAManager().getRatisServer(),
+        scm.getScmMetadataStore().getDeletedBlocksTXTable(),
+        scmHADBTransactionBuffer,
+        scm.getScmContext(),
+        scm.getSequenceIdGen(),
+        metrics);
     blocks = getTransactions(BLOCKS_PER_TXN * 40);
     Assert.assertEquals(0, blocks.size());
-    Assert.assertEquals((long)deletedBlockLog.getCurrentTXID(), 50L);
+    //Assert.assertEquals((long)deletedBlockLog.getCurrentTXID(), 50L);
   }
 
   @Test
@@ -389,10 +441,11 @@ public class TestDeletedBlockLog {
     long containerID;
 
     // Creates {TXNum} TX in the log.
-    for (Map.Entry<Long, List<Long>> entry : generateData(txNum).entrySet()) {
+    Map<Long, List<Long>> deletedBlocks = generateData(txNum);
+    addTransactions(deletedBlocks, true);
+    for (Map.Entry<Long, List<Long>> entry :deletedBlocks.entrySet()) {
       count++;
       containerID = entry.getKey();
-      deletedBlockLog.addTransaction(containerID, entry.getValue());
 
       if (count % 2 == 0) {
         mockContainerInfo(containerID, dnId1);
@@ -415,7 +468,9 @@ public class TestDeletedBlockLog {
     builder.setTxID(11);
     builder.setContainerID(containerID);
     builder.setCount(0);
-    deletedBlockLog.addTransaction(containerID, new LinkedList<>());
+    Map<Long, List<Long>> deletedBlocksMap = new HashMap<>();
+    deletedBlocksMap.put(containerID, new LinkedList<>());
+    addTransactions(deletedBlocksMap, true);
 
     // get should return two transactions for the same container
     blocks = getTransactions(txNum);
@@ -426,21 +481,21 @@ public class TestDeletedBlockLog {
       throws IOException {
     List<DatanodeDetails> dns = Collections.singletonList(dd);
     Pipeline pipeline = Pipeline.newBuilder()
-            .setType(ReplicationType.STAND_ALONE)
-            .setFactor(ReplicationFactor.ONE)
-            .setState(Pipeline.PipelineState.OPEN)
+        .setReplicationConfig(
+            new StandaloneReplicationConfig(ReplicationFactor.ONE))
+        .setState(Pipeline.PipelineState.OPEN)
             .setId(PipelineID.randomId())
             .setNodes(dns)
             .build();
 
     ContainerInfo.Builder builder = new ContainerInfo.Builder();
-    builder.setPipelineID(pipeline.getId())
-        .setReplicationType(pipeline.getType())
-        .setReplicationFactor(pipeline.getFactor());
+    builder.setContainerID(containerID)
+        .setPipelineID(pipeline.getId())
+        .setReplicationConfig(pipeline.getReplicationConfig());
 
     ContainerInfo containerInfo = builder.build();
     Mockito.doReturn(containerInfo).when(containerManager)
-        .getContainer(ContainerID.valueof(containerID));
+        .getContainer(ContainerID.valueOf(containerID));
 
     final Set<ContainerReplica> replicaSet = dns.stream()
         .map(datanodeDetails -> ContainerReplica.newBuilder()
@@ -450,7 +505,7 @@ public class TestDeletedBlockLog {
             .build())
         .collect(Collectors.toSet());
     when(containerManager.getContainerReplicas(
-        ContainerID.valueof(containerID)))
+        ContainerID.valueOf(containerID)))
         .thenReturn(replicaSet);
   }
 }
