@@ -86,6 +86,11 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
   private long quotaInBytes;
   private long quotaInNamespace;
 
+  /**
+   * Bucket Layout.
+   */
+  private BucketLayout bucketLayout;
+
   /**
    * Private constructor, constructed via builder.
    * @param volumeName - Volume name.
@@ -102,6 +107,7 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
    * @param usedBytes - Bucket Quota Usage in bytes.
    * @param quotaInBytes Bucket quota in bytes.
    * @param quotaInNamespace Bucket quota in counts.
+   * @param bucketLayout Bucket Layout.
    */
   @SuppressWarnings("checkstyle:ParameterNumber")
   private OmBucketInfo(String volumeName,
@@ -120,7 +126,8 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
       long usedBytes,
       long usedNamespace,
       long quotaInBytes,
-      long quotaInNamespace) {
+      long quotaInNamespace,
+      BucketLayout bucketLayout) {
     this.volumeName = volumeName;
     this.bucketName = bucketName;
     this.acls = acls;
@@ -138,6 +145,7 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
     this.usedNamespace = usedNamespace;
     this.quotaInBytes = quotaInBytes;
     this.quotaInNamespace = quotaInNamespace;
+    this.bucketLayout = bucketLayout;
   }
 
   /**
@@ -235,6 +243,14 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
     return bekInfo;
   }
 
+  /**
+   * Returns the Bucket Layout.
+   * @return BucketLayout.
+   */
+  public BucketLayout getBucketLayout() {
+    return bucketLayout;
+  }
+
   public String getSourceVolume() {
     return sourceVolume;
   }
@@ -345,7 +361,8 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
         .setUsedBytes(usedBytes)
         .setUsedNamespace(usedNamespace)
         .setQuotaInBytes(quotaInBytes)
-        .setQuotaInNamespace(quotaInNamespace);
+        .setQuotaInNamespace(quotaInNamespace)
+        .setBucketLayout(bucketLayout);
   }
 
   /**
@@ -369,6 +386,7 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
     private long usedNamespace;
     private long quotaInBytes;
     private long quotaInNamespace;
+    private BucketLayout bucketLayout;
 
     public Builder() {
       //Default values
@@ -378,6 +396,7 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
       this.metadata = new HashMap<>();
       this.quotaInBytes = OzoneConsts.QUOTA_RESET;
       this.quotaInNamespace = OzoneConsts.QUOTA_RESET;
+      this.bucketLayout = BucketLayout.DEFAULT;
     }
 
     public Builder setVolumeName(String volume) {
@@ -397,6 +416,10 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
       return this;
     }
 
+    public List<OzoneAcl> getAcls() {
+      return acls;
+    }
+
     public Builder addAcl(OzoneAcl ozoneAcl) {
       if (ozoneAcl != null) {
         this.acls.add(ozoneAcl);
@@ -482,6 +505,11 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
       return this;
     }
 
+    public Builder setBucketLayout(BucketLayout type) {
+      this.bucketLayout = type;
+      return this;
+    }
+
     /**
      * Constructs the OmBucketInfo.
      * @return instance of OmBucketInfo.
@@ -496,7 +524,7 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
       return new OmBucketInfo(volumeName, bucketName, acls, isVersionEnabled,
           storageType, creationTime, modificationTime, objectID, updateID,
           metadata, bekInfo, sourceVolume, sourceBucket, usedBytes,
-          usedNamespace, quotaInBytes, quotaInNamespace);
+          usedNamespace, quotaInBytes, quotaInNamespace, bucketLayout);
     }
   }
 
@@ -519,6 +547,9 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
         .addAllMetadata(KeyValueUtil.toProtobuf(metadata))
         .setQuotaInBytes(quotaInBytes)
         .setQuotaInNamespace(quotaInNamespace);
+    if (bucketLayout != null) {
+      bib.setBucketLayout(bucketLayout.toProto());
+    }
     if (bekInfo != null && bekInfo.getKeyName() != null) {
       bib.setBeinfo(OMPBHelper.convert(bekInfo));
     }
@@ -531,13 +562,24 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
     return bib.build();
   }
 
+
   /**
    * Parses BucketInfo protobuf and creates OmBucketInfo.
    * @param bucketInfo
    * @return instance of OmBucketInfo
    */
   public static OmBucketInfo getFromProtobuf(BucketInfo bucketInfo) {
-    OmBucketInfo.Builder obib = OmBucketInfo.newBuilder()
+    return getFromProtobuf(bucketInfo, null);
+  }
+
+  /**
+   * Parses BucketInfo protobuf and creates OmBucketInfo.
+   * @param bucketInfo
+   * @return instance of OmBucketInfo
+   */
+  public static OmBucketInfo getFromProtobuf(BucketInfo bucketInfo,
+      BucketLayout buckLayout) {
+    Builder obib = OmBucketInfo.newBuilder()
         .setVolumeName(bucketInfo.getVolumeName())
         .setBucketName(bucketInfo.getBucketName())
         .setAcls(bucketInfo.getAclsList().stream().map(
@@ -550,6 +592,12 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
         .setQuotaInBytes(bucketInfo.getQuotaInBytes())
         .setUsedNamespace(bucketInfo.getUsedNamespace())
         .setQuotaInNamespace(bucketInfo.getQuotaInNamespace());
+    if (buckLayout != null) {
+      obib.setBucketLayout(buckLayout);
+    } else if (bucketInfo.getBucketLayout() != null) {
+      obib.setBucketLayout(
+          BucketLayout.fromProto(bucketInfo.getBucketLayout()));
+    }
     if (bucketInfo.hasObjectID()) {
       obib.setObjectID(bucketInfo.getObjectID());
     }
@@ -587,7 +635,8 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
         ", usedBytes='" + usedBytes + "'" +
         ", usedNamespace='" + usedNamespace + "'" +
         ", quotaInBytes='" + quotaInBytes + "'" +
-        ", quotaInNamespace='" + quotaInNamespace + '\'' +
+        ", quotaInNamespace='" + quotaInNamespace + "'" +
+        ", bucketLayout='" + bucketLayout + '\'' +
         sourceInfo +
         '}';
   }
@@ -642,6 +691,7 @@ public final class OmBucketInfo extends WithObjectID implements Auditable {
         ", usedNamespace=" + usedNamespace +
         ", quotaInBytes=" + quotaInBytes +
         ", quotaInNamespace=" + quotaInNamespace +
+        ", bucketLayout=" + bucketLayout +
         '}';
   }
 }
