@@ -13,9 +13,25 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+
 DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
 cd "$DIR/../../.." || exit 1
 
+: ${KUBECONFIG:=/etc/rancher/k3s/k3s.yaml}
+
+export KUBECONFIG
+
+source "${DIR}/_lib.sh"
+
+install_flekszible
+install_virtualenv
+install_robot
+if [[ "$(uname -s)" = "Darwin" ]]; then
+  echo "Skip installing k3s, not supported on Mac.  Make sure a working Kubernetes cluster is available." >&2
+else
+  install_k3s
+fi
+
 REPORT_DIR=${OUTPUT_DIR:-"$DIR/../../../target/kubernetes"}
 
 OZONE_VERSION=$(mvn help:evaluate -Dexpression=ozone.version -q -DforceStdout)
