@@ -25,7 +25,7 @@ import org.apache.hadoop.ozone.MiniOzoneCluster;
 import org.apache.hadoop.ozone.client.ObjectStore;
 import org.apache.hadoop.ozone.client.OzoneClientFactory;
 import org.apache.hadoop.ozone.client.OzoneVolume;
-import org.apache.hadoop.test.GenericTestUtils;
+import org.apache.ozone.test.GenericTestUtils;
 import org.apache.ratis.server.RaftServer;
 import org.apache.ratis.server.raftlog.RaftLog;
 import java.util.LinkedList;
@@ -40,6 +40,8 @@ import java.io.FileOutputStream;
 import java.io.IOException;
 import java.net.URI;
 
+import static org.apache.ozone.test.GenericTestUtils.getTempPath;
+
 /**
  * Test for HadoopNestedDirGenerator.
  */
@@ -54,8 +56,7 @@ public class TestHadoopNestedDirGenerator {
           LoggerFactory.getLogger(TestHadoopNestedDirGenerator.class);
   @Before
     public void setup() {
-    path = GenericTestUtils
-                .getTempPath(TestOzoneClientKeyGenerator.class.getSimpleName());
+    path = getTempPath(TestHadoopNestedDirGenerator.class.getSimpleName());
     GenericTestUtils.setLogLevel(RaftLog.LOG, Level.DEBUG);
     GenericTestUtils.setLogLevel(RaftServer.LOG, Level.DEBUG);
     File baseDir = new File(path);
