@@ -30,4 +30,8 @@ public interface OMStarterInterface {
       AuthenticationException;
   boolean init(OzoneConfiguration conf) throws IOException,
       AuthenticationException;
+  void bootstrap(OzoneConfiguration conf) throws IOException,
+      AuthenticationException;
+  void startAndCancelPrepare(OzoneConfiguration conf) throws IOException,
+      AuthenticationException;
 }
\ No newline at end of file
