@@ -18,41 +18,46 @@
 
 package org.apache.hadoop.ozone.om.ratis;
 
+import com.google.common.annotations.VisibleForTesting;
+import com.google.common.base.Preconditions;
+import com.google.common.collect.Lists;
+import com.google.protobuf.InvalidProtocolBufferException;
+import com.google.protobuf.ServiceException;
+
 import java.io.File;
 import java.io.IOException;
 import java.net.InetSocketAddress;
 import java.nio.charset.StandardCharsets;
-import java.nio.file.Paths;
+import java.security.cert.X509Certificate;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 import java.util.Map;
 import java.util.UUID;
+import java.util.concurrent.atomic.AtomicLong;
 import java.util.concurrent.TimeUnit;
 
-import com.google.common.base.Preconditions;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.conf.StorageUnit;
 import org.apache.hadoop.hdds.security.x509.SecurityConfig;
 import org.apache.hadoop.hdds.security.x509.certificate.client.CertificateClient;
-import org.apache.hadoop.hdds.server.ServerUtils;
 import org.apache.hadoop.hdds.tracing.TracingUtil;
+import org.apache.hadoop.hdds.utils.HAUtils;
 import org.apache.hadoop.ipc.ProtobufRpcEngine.Server;
 import org.apache.hadoop.ozone.om.OMConfigKeys;
 import org.apache.hadoop.ozone.om.OzoneManager;
 import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.ozone.om.exceptions.OMLeaderNotReadyException;
 import org.apache.hadoop.ozone.om.exceptions.OMNotLeaderException;
-import org.apache.hadoop.ozone.om.ha.OMNodeDetails;
+import org.apache.hadoop.ozone.om.helpers.OMNodeDetails;
 import org.apache.hadoop.ozone.om.helpers.OMRatisHelper;
+import org.apache.hadoop.ozone.om.ratis.utils.OzoneManagerRatisUtils;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMRequest;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.OMResponse;
+import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.Status;
 
-import com.google.common.annotations.VisibleForTesting;
-import com.google.common.base.Strings;
-import com.google.protobuf.InvalidProtocolBufferException;
-import com.google.protobuf.ServiceException;
 import org.apache.ratis.RaftConfigKeys;
 import org.apache.ratis.conf.Parameters;
 import org.apache.ratis.conf.RaftProperties;
@@ -60,6 +65,7 @@ import org.apache.ratis.grpc.GrpcConfigKeys;
 import org.apache.ratis.grpc.GrpcTlsConfig;
 import org.apache.ratis.netty.NettyConfigKeys;
 import org.apache.ratis.protocol.ClientId;
+import org.apache.ratis.protocol.SetConfigurationRequest;
 import org.apache.ratis.protocol.exceptions.LeaderNotReadyException;
 import org.apache.ratis.protocol.exceptions.NotLeaderException;
 import org.apache.ratis.protocol.exceptions.StateMachineException;
@@ -82,12 +88,9 @@ import org.apache.ratis.util.TimeDuration;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import static org.apache.hadoop.hdds.HddsConfigKeys.OZONE_METADATA_DIRS;
 import static org.apache.hadoop.ipc.RpcConstants.DUMMY_CLIENT_ID;
 import static org.apache.hadoop.ipc.RpcConstants.INVALID_CALL_ID;
-import static org.apache.hadoop.ozone.OzoneConsts.OM_RATIS_SNAPSHOT_DIR;
 import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_HA_PREFIX;
-import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_RATIS_SNAPSHOT_DIR;
 
 /**
  * Creates a Ratis server endpoint for OM.
@@ -102,9 +105,130 @@ public final class OzoneManagerRatisServer {
   private final RaftGroupId raftGroupId;
   private final RaftGroup raftGroup;
   private final RaftPeerId raftPeerId;
+  private final List<RaftPeer> raftPeers;
 
   private final OzoneManager ozoneManager;
   private final OzoneManagerStateMachine omStateMachine;
+  private final String ratisStorageDir;
+
+  private final ClientId clientId = ClientId.randomId();
+  private static final AtomicLong CALL_ID_COUNTER = new AtomicLong();
+
+  private static long nextCallId() {
+    return CALL_ID_COUNTER.getAndIncrement() & Long.MAX_VALUE;
+  }
+
+  /**
+   * Returns an OM Ratis server.
+   * @param conf configuration
+   * @param om the OM instance starting the ratis server
+   * @param raftGroupIdStr raft group id string
+   * @param localRaftPeerId raft peer id of this Ratis server
+   * @param addr address of the ratis server
+   * @param peers peer nodes in the raft ring
+   * @throws IOException
+   */
+  @SuppressWarnings({"parameternumber", "java:S107"})
+  private OzoneManagerRatisServer(ConfigurationSource conf, OzoneManager om,
+      String raftGroupIdStr, RaftPeerId localRaftPeerId,
+      InetSocketAddress addr, List<RaftPeer> peers, boolean isBootstrapping,
+      SecurityConfig secConfig, CertificateClient certClient)
+      throws IOException {
+    this.ozoneManager = om;
+    this.omRatisAddress = addr;
+    this.port = addr.getPort();
+    this.ratisStorageDir = OzoneManagerRatisUtils.getOMRatisDirectory(conf);
+    RaftProperties serverProperties = newRaftProperties(conf);
+
+    this.raftPeerId = localRaftPeerId;
+    this.raftGroupId = RaftGroupId.valueOf(
+        getRaftGroupIdFromOmServiceId(raftGroupIdStr));
+    this.raftPeers = Lists.newArrayList();
+    this.raftPeers.addAll(peers);
+    this.raftGroup = RaftGroup.valueOf(raftGroupId, peers);
+
+    if (isBootstrapping) {
+      LOG.info("OM started in Bootstrap mode. Instantiating OM Ratis server " +
+          "with groupID: {}", raftGroupIdStr);
+    } else {
+      StringBuilder raftPeersStr = new StringBuilder();
+      for (RaftPeer peer : peers) {
+        raftPeersStr.append(", ").append(peer.getAddress());
+      }
+      LOG.info("Instantiating OM Ratis server with groupID: {} and peers: {}",
+          raftGroupIdStr, raftPeersStr.toString().substring(2));
+    }
+    this.omStateMachine = getStateMachine(conf);
+
+    Parameters parameters = createServerTlsParameters(secConfig, certClient);
+    this.server = RaftServer.newBuilder()
+        .setServerId(this.raftPeerId)
+        .setGroup(this.raftGroup)
+        .setProperties(serverProperties)
+        .setParameters(parameters)
+        .setStateMachine(omStateMachine)
+        .build();
+  }
+
+  /**
+   * Creates an instance of OzoneManagerRatisServer.
+   */
+  public static OzoneManagerRatisServer newOMRatisServer(
+      ConfigurationSource ozoneConf, OzoneManager omProtocol,
+      OMNodeDetails omNodeDetails, Map<String, OMNodeDetails> peerNodes,
+      SecurityConfig secConfig, CertificateClient certClient,
+      boolean isBootstrapping) throws IOException {
+
+    // RaftGroupId is the omServiceId
+    String omServiceId = omNodeDetails.getServiceId();
+
+    String omNodeId = omNodeDetails.getNodeId();
+    RaftPeerId localRaftPeerId = RaftPeerId.getRaftPeerId(omNodeId);
+
+    InetSocketAddress ratisAddr = new InetSocketAddress(
+        omNodeDetails.getInetAddress(), omNodeDetails.getRatisPort());
+
+    RaftPeer localRaftPeer = RaftPeer.newBuilder()
+        .setId(localRaftPeerId)
+        .setAddress(ratisAddr)
+        .build();
+
+    // If OM is started in bootstrap mode, do not add peers to the RaftGroup.
+    // Raft peers will be added after SetConfiguration transaction is
+    // committed by leader and propagated to followers.
+    List<RaftPeer> raftPeers = new ArrayList<>();
+    if (!isBootstrapping) {
+      // On regular startup, add all OMs to Ratis ring
+      raftPeers.add(localRaftPeer);
+
+      for (Map.Entry<String, OMNodeDetails> peerInfo : peerNodes.entrySet()) {
+        String peerNodeId = peerInfo.getKey();
+        OMNodeDetails peerNode = peerInfo.getValue();
+        RaftPeerId raftPeerId = RaftPeerId.valueOf(peerNodeId);
+        RaftPeer raftPeer;
+        if (peerNode.isHostUnresolved()) {
+          raftPeer = RaftPeer.newBuilder()
+              .setId(raftPeerId)
+              .setAddress(peerNode.getRatisHostPortStr())
+              .build();
+        } else {
+          InetSocketAddress peerRatisAddr = new InetSocketAddress(
+              peerNode.getInetAddress(), peerNode.getRatisPort());
+          raftPeer = RaftPeer.newBuilder()
+              .setId(raftPeerId)
+              .setAddress(peerRatisAddr)
+              .build();
+        }
+
+        // Add other OM nodes belonging to the same OM service to the Ratis ring
+        raftPeers.add(raftPeer);
+      }
+    }
+
+    return new OzoneManagerRatisServer(ozoneConf, omProtocol, omServiceId,
+        localRaftPeerId, ratisAddr, raftPeers, isBootstrapping, secConfig,
+        certClient);
+  }
 
   /**
    * Submit request to Ratis server.
@@ -113,10 +237,29 @@ public final class OzoneManagerRatisServer {
    * @throws ServiceException
    */
   public OMResponse submitRequest(OMRequest omRequest) throws ServiceException {
-    RaftClientRequest raftClientRequest =
-        createWriteRaftClientRequest(omRequest);
-    RaftClientReply raftClientReply = submitRequestToRatis(raftClientRequest);
-    return processReply(omRequest, raftClientReply);
+    // In prepare mode, only prepare and cancel requests are allowed to go
+    // through.
+    if (ozoneManager.getPrepareState().requestAllowed(omRequest.getCmdType())) {
+      RaftClientRequest raftClientRequest =
+          createWriteRaftClientRequest(omRequest);
+      RaftClientReply raftClientReply = submitRequestToRatis(raftClientRequest);
+
+      return processReply(omRequest, raftClientReply);
+    } else {
+      LOG.info("Rejecting write request on OM {} because it is in prepare " +
+          "mode: {}", ozoneManager.getOMNodeId(),
+          omRequest.getCmdType().name());
+
+      String message = "Cannot apply write request " +
+          omRequest.getCmdType().name() + " when OM is in prepare mode.";
+      OMResponse.Builder omResponse = OMResponse.newBuilder()
+          .setMessage(message)
+          .setStatus(Status.NOT_SUPPORTED_OPERATION_WHEN_PREPARED)
+          .setCmdType(omRequest.getCmdType())
+          .setTraceID(omRequest.getTraceID())
+          .setSuccess(false);
+      return omResponse.build();
+    }
   }
 
   /**
@@ -143,6 +286,93 @@ public final class OzoneManagerRatisServer {
     }
   }
 
+  /**
+   * Add new OM to the Ratis ring.
+   */
+  public void addOMToRatisRing(OMNodeDetails newOMNode) throws IOException {
+
+    Preconditions.checkNotNull(newOMNode);
+
+    String newOMNodeId = newOMNode.getNodeId();
+    RaftPeerId newOMRaftPeerId = RaftPeerId.valueOf(newOMNodeId);
+    InetSocketAddress newOMRatisAddr = new InetSocketAddress(
+        newOMNode.getHostAddress(), newOMNode.getRatisPort());
+    RaftPeer newRaftPeer = RaftPeer.newBuilder()
+        .setId(newOMRaftPeerId)
+        .setAddress(newOMRatisAddr)
+        .build();
+
+    LOG.info("{}: Submitting SetConfiguration request to Ratis server to add" +
+            " new OM peer {} to the Ratis group {}", ozoneManager.getOMNodeId(),
+        newRaftPeer, raftGroup);
+
+    List<RaftPeer> newPeersList = new ArrayList<>();
+    newPeersList.addAll(raftPeers);
+    newPeersList.add(newRaftPeer);
+
+    checkLeaderStatus();
+    SetConfigurationRequest request = new SetConfigurationRequest(clientId,
+        server.getId(), raftGroupId, nextCallId(), newPeersList);
+
+    try {
+      RaftClientReply raftClientReply = server.setConfiguration(request);
+      if (raftClientReply.isSuccess()) {
+        LOG.info("Added OM {} to Ratis group {}.", newOMNodeId, raftGroupId);
+      } else {
+        LOG.error("Failed to add OM {} to Ratis group {}. Ratis " +
+                "SetConfiguration reply: {}", newOMNodeId, raftGroupId,
+            raftClientReply);
+        throw new IOException("Failed to add OM " + newOMNodeId + " to Ratis " +
+            "ring.");
+      }
+    } catch (IOException e) {
+      LOG.error("Failed to update Ratis configuration and add OM {} to " +
+          "Ratis group {}", newOMNodeId, raftGroupId, e);
+      throw e;
+    }
+  }
+
+  /**
+   * Return a list of peer NodeIds.
+   */
+  public List<String> getPeerIds() {
+    List<String> peerIds = new ArrayList<>();
+    for (RaftPeer raftPeer : raftPeers) {
+      peerIds.add(raftPeer.getId().toString());
+    }
+    return peerIds;
+  }
+
+  /**
+   * Check if the input peerId exists in the peers list.
+   * @return true if the nodeId is self or it exists in peer node list,
+   *         false otherwise.
+   */
+  @VisibleForTesting
+  public boolean doesPeerExist(String peerId) {
+    for (RaftPeer raftPeer : raftPeers) {
+      if (raftPeer.getId().toString().equals(peerId)) {
+        return true;
+      }
+    }
+    return false;
+  }
+
+  /**
+   * Add given node to list of RaftPeers.
+   */
+  public void addRaftPeer(OMNodeDetails omNodeDetails) {
+    InetSocketAddress newOMRatisAddr = new InetSocketAddress(
+        omNodeDetails.getHostAddress(), omNodeDetails.getRatisPort());
+
+    raftPeers.add(RaftPeer.newBuilder()
+        .setId(RaftPeerId.valueOf(omNodeDetails.getNodeId()))
+        .setAddress(newOMRatisAddr)
+        .build());
+
+    LOG.info("Added OM {} to Ratis Peers list.", omNodeDetails.getNodeId());
+  }
+
   /**
    * Create Write RaftClient request from OMRequest.
    * @param omRequest
@@ -251,106 +481,6 @@ public final class OzoneManagerRatisServer {
     }
   }
 
-
-  /**
-   * Returns an OM Ratis server.
-   * @param conf configuration
-   * @param om the OM instance starting the ratis server
-   * @param raftGroupIdStr raft group id string
-   * @param localRaftPeerId raft peer id of this Ratis server
-   * @param addr address of the ratis server
-   * @param raftPeers peer nodes in the raft ring
-   * @throws IOException
-   */
-  @SuppressWarnings({"parameternumber", "java:S107"})
-  private OzoneManagerRatisServer(ConfigurationSource conf,
-      OzoneManager om,
-      String raftGroupIdStr, RaftPeerId localRaftPeerId,
-      InetSocketAddress addr, List<RaftPeer> raftPeers,
-      SecurityConfig secConfig, CertificateClient certClient)
-      throws IOException {
-    this.ozoneManager = om;
-    this.omRatisAddress = addr;
-    this.port = addr.getPort();
-    RaftProperties serverProperties = newRaftProperties(conf);
-
-    this.raftPeerId = localRaftPeerId;
-    this.raftGroupId = RaftGroupId.valueOf(
-        getRaftGroupIdFromOmServiceId(raftGroupIdStr));
-    this.raftGroup = RaftGroup.valueOf(raftGroupId, raftPeers);
-
-    StringBuilder raftPeersStr = new StringBuilder();
-    for (RaftPeer peer : raftPeers) {
-      raftPeersStr.append(", ").append(peer.getAddress());
-    }
-    LOG.info("Instantiating OM Ratis server with GroupID: {} and " +
-        "Raft Peers: {}", raftGroupIdStr, raftPeersStr.toString().substring(2));
-
-    this.omStateMachine = getStateMachine(conf);
-
-    Parameters parameters = createServerTlsParameters(secConfig, certClient);
-    this.server = RaftServer.newBuilder()
-        .setServerId(this.raftPeerId)
-        .setGroup(this.raftGroup)
-        .setProperties(serverProperties)
-        .setParameters(parameters)
-        .setStateMachine(omStateMachine)
-        .build();
-  }
-
-  /**
-   * Creates an instance of OzoneManagerRatisServer.
-   */
-  public static OzoneManagerRatisServer newOMRatisServer(
-      ConfigurationSource ozoneConf, OzoneManager omProtocol,
-      OMNodeDetails omNodeDetails, List<OMNodeDetails> peerNodes,
-      SecurityConfig secConfig, CertificateClient certClient)
-      throws IOException {
-
-    // RaftGroupId is the omServiceId
-    String omServiceId = omNodeDetails.getOMServiceId();
-
-    String omNodeId = omNodeDetails.getOMNodeId();
-    RaftPeerId localRaftPeerId = RaftPeerId.getRaftPeerId(omNodeId);
-
-    InetSocketAddress ratisAddr = new InetSocketAddress(
-        omNodeDetails.getInetAddress(), omNodeDetails.getRatisPort());
-
-    RaftPeer localRaftPeer = RaftPeer.newBuilder()
-        .setId(localRaftPeerId)
-        .setAddress(ratisAddr)
-        .build();
-
-    List<RaftPeer> raftPeers = new ArrayList<>();
-    // Add this Ratis server to the Ratis ring
-    raftPeers.add(localRaftPeer);
-
-    for (OMNodeDetails peerInfo : peerNodes) {
-      String peerNodeId = peerInfo.getOMNodeId();
-      RaftPeerId raftPeerId = RaftPeerId.valueOf(peerNodeId);
-      RaftPeer raftPeer;
-      if (peerInfo.isHostUnresolved()) {
-        raftPeer = RaftPeer.newBuilder()
-            .setId(raftPeerId)
-            .setAddress(peerInfo.getRatisHostPortStr())
-            .build();
-      } else {
-        InetSocketAddress peerRatisAddr = new InetSocketAddress(
-            peerInfo.getInetAddress(), peerInfo.getRatisPort());
-        raftPeer = RaftPeer.newBuilder()
-            .setId(raftPeerId)
-            .setAddress(peerRatisAddr)
-            .build();
-      }
-
-      // Add other OM nodes belonging to the same OM service to the Ratis ring
-      raftPeers.add(raftPeer);
-    }
-
-    return new OzoneManagerRatisServer(ozoneConf, omProtocol, omServiceId,
-        localRaftPeerId, ratisAddr, raftPeers, secConfig, certClient);
-  }
-
   public RaftGroup getRaftGroup() {
     return this.raftGroup;
   }
@@ -417,14 +547,15 @@ public final class OzoneManagerRatisServer {
     }
 
     // Set Ratis storage directory
-    String storageDir = OzoneManagerRatisServer.getOMRatisDirectory(conf);
     RaftServerConfigKeys.setStorageDir(properties,
-        Collections.singletonList(new File(storageDir)));
-    // Disable the pre vote feature in Ratis
-    RaftServerConfigKeys.LeaderElection.setPreVote(properties, false);
+        Collections.singletonList(new File(ratisStorageDir)));
+    // Disable/enable the pre vote feature in Ratis
+    RaftServerConfigKeys.LeaderElection.setPreVote(properties,
+        conf.getBoolean(OMConfigKeys.OZONE_OM_RATIS_SERVER_ELECTION_PRE_VOTE,
+            OMConfigKeys.OZONE_OM_RATIS_SERVER_ELECTION_PRE_VOTE_DEFAULT));
 
     // Set RAFT segment size
-    final int raftSegmentSize = (int) conf.getStorageSize(
+    final long raftSegmentSize = (long) conf.getStorageSize(
         OMConfigKeys.OZONE_OM_RATIS_SEGMENT_SIZE_KEY,
         OMConfigKeys.OZONE_OM_RATIS_SEGMENT_SIZE_DEFAULT,
         StorageUnit.BYTES);
@@ -433,7 +564,7 @@ public final class OzoneManagerRatisServer {
     RaftServerConfigKeys.Log.setPurgeUptoSnapshotIndex(properties, true);
 
     // Set RAFT segment pre-allocated size
-    final int raftSegmentPreallocatedSize = (int) conf.getStorageSize(
+    final long raftSegmentPreallocatedSize = (long) conf.getStorageSize(
         OMConfigKeys.OZONE_OM_RATIS_SEGMENT_PREALLOCATED_SIZE_KEY,
         OMConfigKeys.OZONE_OM_RATIS_SEGMENT_PREALLOCATED_SIZE_DEFAULT,
         StorageUnit.BYTES);
@@ -594,6 +725,24 @@ public final class OzoneManagerRatisServer {
     return RaftServerStatus.NOT_LEADER;
   }
 
+  /**
+   * Get list of peer NodeIds from Ratis.
+   * @return List of Peer NodeId's.
+   */
+  @VisibleForTesting
+  public List<String> getCurrentPeersFromRaftConf() throws IOException {
+    try {
+      Collection<RaftPeer> currentPeers =
+          server.getDivision(raftGroupId).getRaftConf().getCurrentPeers();
+      List<String> currentPeerList = new ArrayList<>();
+      currentPeers.forEach(e -> currentPeerList.add(e.getId().toString()));
+      return currentPeerList;
+    } catch (IOException e) {
+      // In this case we return not a leader.
+      throw new IOException("Failed to get peer information from Ratis.", e);
+    }
+  }
+
   public int getServerPort() {
     return port;
   }
@@ -608,34 +757,12 @@ public final class OzoneManagerRatisServer {
     return this.raftPeerId;
   }
 
-  private UUID getRaftGroupIdFromOmServiceId(String omServiceId) {
+  public static UUID getRaftGroupIdFromOmServiceId(String omServiceId) {
     return UUID.nameUUIDFromBytes(omServiceId.getBytes(StandardCharsets.UTF_8));
   }
 
-  /**
-   * Get the local directory where ratis logs will be stored.
-   */
-  public static String getOMRatisDirectory(ConfigurationSource conf) {
-    String storageDir = conf.get(OMConfigKeys.OZONE_OM_RATIS_STORAGE_DIR);
-
-    if (Strings.isNullOrEmpty(storageDir)) {
-      storageDir = ServerUtils.getDefaultRatisDirectory(conf);
-    }
-    return storageDir;
-  }
-
-  public static String getOMRatisSnapshotDirectory(ConfigurationSource conf) {
-    String snapshotDir = conf.get(OZONE_OM_RATIS_SNAPSHOT_DIR);
-
-    // If ratis snapshot directory is not set, fall back to ozone.metadata.dir.
-    if (Strings.isNullOrEmpty(snapshotDir)) {
-      LOG.warn("{} is not configured. Falling back to {} config",
-          OZONE_OM_RATIS_SNAPSHOT_DIR, OZONE_METADATA_DIRS);
-      File metaDirPath = ServerUtils.getOzoneMetaDirPath(conf);
-      snapshotDir = Paths.get(metaDirPath.getPath(),
-          OM_RATIS_SNAPSHOT_DIR).toString();
-    }
-    return snapshotDir;
+  public String getRatisStorageDir() {
+    return ratisStorageDir;
   }
 
   public TermIndex getLastAppliedTermIndex() {
@@ -647,13 +774,15 @@ public final class OzoneManagerRatisServer {
   }
 
   private static Parameters createServerTlsParameters(SecurityConfig conf,
-      CertificateClient caClient) {
+      CertificateClient caClient) throws IOException {
     Parameters parameters = new Parameters();
 
     if (conf.isSecurityEnabled() && conf.isGrpcTlsEnabled()) {
+      List<X509Certificate> caList = HAUtils.buildCAX509List(caClient,
+          conf.getConfiguration());
       GrpcTlsConfig config = new GrpcTlsConfig(
           caClient.getPrivateKey(), caClient.getCertificate(),
-          caClient.getCACertificate(), true);
+          caList, true);
       GrpcConfigKeys.Server.setTlsConf(parameters, config);
       GrpcConfigKeys.Admin.setTlsConf(parameters, config);
       GrpcConfigKeys.Client.setTlsConf(parameters, config);
