@@ -72,11 +72,15 @@ public class S3GetSecretRequest extends OMClientRequest {
     // Generate S3 Secret to be used by OM quorum.
     String kerberosID = s3GetSecretRequest.getKerberosID();
 
-    UserGroupInformation user = ProtobufRpcEngine.Server.getRemoteUser();
-    if (!user.getUserName().equals(kerberosID)) {
-      throw new OMException("User mismatch. Requested user name is " +
-          "mismatched " + kerberosID +", with current user " +
-          user.getUserName(), OMException.ResultCodes.USER_MISMATCH);
+    final UserGroupInformation ugi = ProtobufRpcEngine.Server.getRemoteUser();
+    final String username = ugi.getUserName();
+    // Permission check. Users need to be themselves or have admin privilege
+    if (!username.equals(kerberosID) &&
+        !ozoneManager.isAdmin(ugi)) {
+      throw new OMException("Requested user name '" + kerberosID +
+          "' doesn't match current user '" + username +
+          "', nor does current user has administrator privilege.",
+          OMException.ResultCodes.USER_MISMATCH);
     }
 
     String s3Secret = DigestUtils.sha256Hex(OmUtils.getSHADigest());
@@ -186,4 +190,5 @@ public class S3GetSecretRequest extends OMClientRequest {
     }
     return omClientResponse;
   }
+
 }
