@@ -53,6 +53,7 @@ import org.apache.hadoop.ozone.client.OzoneClientFactory;
 import org.apache.hadoop.ozone.client.OzoneVolume;
 import org.apache.hadoop.ozone.client.io.OzoneInputStream;
 import org.apache.hadoop.ozone.client.io.OzoneOutputStream;
+import org.apache.hadoop.ozone.util.ShutdownHookManager;
 import org.apache.hadoop.util.Time;
 import org.apache.hadoop.util.VersionInfo;
 
@@ -74,6 +75,8 @@ import picocli.CommandLine.Command;
 import picocli.CommandLine.Option;
 import picocli.CommandLine.ParentCommand;
 
+import static org.apache.hadoop.ozone.conf.OzoneServiceConfig.DEFAULT_SHUTDOWN_HOOK_PRIORITY;
+
 /**
  * Data generator tool to generate as much keys as possible.
  */
@@ -189,6 +192,13 @@ public final class RandomKeyGenerator implements Callable<Void> {
   )
   private String omServiceID = null;
 
+  @Option(
+      names = "--clean-objects",
+      description = "Specifies whether to clean the random generated " +
+          "volumes, buckets and keys."
+  )
+  private boolean cleanObjects = false;
+
   private int threadPoolSize;
 
   private OzoneClient ozoneClient;
@@ -217,6 +227,10 @@ public final class RandomKeyGenerator implements Callable<Void> {
   private AtomicInteger numberOfBucketsCreated;
   private AtomicLong numberOfKeysAdded;
 
+  private AtomicInteger cleanedBucketCounter;
+  private AtomicInteger numberOfBucketsCleaned;
+  private AtomicInteger numberOfVolumesCleaned;
+
   private Long totalWritesValidated;
   private Long writeValidationSuccessCount;
   private Long writeValidationFailureCount;
@@ -251,6 +265,9 @@ public final class RandomKeyGenerator implements Callable<Void> {
     keyCounter = new AtomicLong();
     volumes = new ConcurrentHashMap<>();
     buckets = new ConcurrentHashMap<>();
+    cleanedBucketCounter = new AtomicInteger();
+    numberOfBucketsCleaned = new AtomicInteger();
+    numberOfVolumesCleaned = new AtomicInteger();
     if (omServiceID != null) {
       ozoneClient = OzoneClientFactory.getRpcClient(omServiceID, configuration);
     } else {
@@ -307,6 +324,7 @@ public final class RandomKeyGenerator implements Callable<Void> {
     LOG.info("Key size: {} bytes", keySize);
     LOG.info("Buffer size: {} bytes", bufferSize);
     LOG.info("validateWrites : {}", validateWrites);
+    LOG.info("cleanObjects : {}", cleanObjects);
     for (int i = 0; i < numOfThreads; i++) {
       executor.execute(new ObjectCreator());
     }
@@ -352,6 +370,9 @@ public final class RandomKeyGenerator implements Callable<Void> {
     if (validator != null) {
       validator.join();
     }
+    if (cleanObjects && exception == null) {
+      doCleanObjects();
+    }
     ozoneClient.close();
     if (exception != null) {
       throw new RuntimeException(exception);
@@ -363,14 +384,56 @@ public final class RandomKeyGenerator implements Callable<Void> {
    * Adds ShutdownHook to print statistics.
    */
   private void addShutdownHook() {
-    Runtime.getRuntime().addShutdownHook(
-        new Thread(() -> {
-          printStats(System.out);
-          if (freon != null) {
-            freon.stopHttpServer();
-          }
-        }));
+    ShutdownHookManager.get().addShutdownHook(() -> {
+      printStats(System.out);
+      if (freon != null) {
+        freon.stopHttpServer();
+      }
+    }, DEFAULT_SHUTDOWN_HOOK_PRIORITY);
   }
+
+  private void doCleanObjects() throws InterruptedException {
+    // Clean Buckets first
+    executor = Executors.newFixedThreadPool(threadPoolSize);
+    for (int i = 0; i < numOfThreads; i++) {
+      executor.execute(new BucketCleaner());
+    }
+    LongSupplier currentValue = numberOfBucketsCleaned::get;
+    progressbar = new ProgressBar(System.out, totalBucketCount, currentValue);
+
+    LOG.info("Starting clean progress bar Thread.");
+    progressbar.start();
+
+    try {
+      // wait until all Buckets are cleaned or exception occurred.
+      while ((numberOfBucketsCleaned.get() != totalBucketCount)
+          && exception == null) {
+        try {
+          Thread.sleep(CHECK_INTERVAL_MILLIS);
+        } catch (InterruptedException e) {
+          throw e;
+        }
+      }
+    } catch (InterruptedException e) {
+      LOG.error("Failed to wait until all Buckets are cleaned", e);
+      Thread.currentThread().interrupt();
+    }
+
+    executor.shutdown();
+    executor.awaitTermination(Integer.MAX_VALUE, TimeUnit.MILLISECONDS);
+
+    // Clean Volume after cleaning Bucket
+    for (int v = 0; v < numOfVolumes; v++) {
+      cleanVolume(v);
+    }
+
+    if (exception != null) {
+      progressbar.terminate();
+    } else {
+      progressbar.shutdown();
+    }
+  }
+
   /**
    * Prints stats of {@link Freon} run to the PrintStream.
    *
@@ -526,6 +589,25 @@ public final class RandomKeyGenerator implements Callable<Void> {
     return numberOfKeysAdded.get();
   }
 
+  /**
+   * Returns the number of volumes cleaned.
+   *
+   * @return cleaned volume count.
+   */
+  @VisibleForTesting
+  int getNumberOfVolumesCleaned() {
+    return numberOfVolumesCleaned.get();
+  }
+
+  /**
+   * Returns the number of buckets cleaned.
+   *
+   * @return cleaned bucket count.
+   */
+  @VisibleForTesting
+  int getNumberOfBucketsCleaned() {
+    return numberOfBucketsCleaned.get();
+  }
   /**
    * Returns true if random validation of write is enabled.
    *
@@ -625,6 +707,18 @@ public final class RandomKeyGenerator implements Callable<Void> {
     }
   }
 
+  private class BucketCleaner implements Runnable {
+    @Override
+    public void run() {
+      int b;
+      while ((b = cleanedBucketCounter.getAndIncrement()) < totalBucketCount) {
+        if (!cleanBucket(b)) {
+          return;
+        }
+      }
+    }
+  }
+
   private boolean createVolume(int volumeNumber) {
     String volumeName = "vol-" + volumeNumber + "-"
         + RandomStringUtils.randomNumeric(5);
@@ -744,6 +838,47 @@ public final class RandomKeyGenerator implements Callable<Void> {
     }
   }
 
+  private boolean cleanVolume(int volumeNumber) {
+    OzoneVolume volume = getVolume(volumeNumber);
+    String volumeName = volume.getName();
+    LOG.trace("Cleaning volume: {}", volumeName);
+    try (AutoCloseable scope = TracingUtil
+        .createActivatedSpan("cleanVolume")) {
+      objectStore.deleteVolume(volumeName);
+      numberOfVolumesCleaned.getAndIncrement();
+      return true;
+    } catch (Throwable e) {
+      exception = e;
+      LOG.error("Could not clean volume", e);
+      return false;
+    }
+  }
+
+  private boolean cleanBucket(int globalBucketNumber) {
+    int volumeNumber = globalBucketNumber % numOfVolumes;
+    OzoneVolume volume = getVolume(volumeNumber);
+    OzoneBucket bucket = getBucket(globalBucketNumber);
+    String bucketName = bucket.getName();
+    if (volume == null) {
+      LOG.error("Could not find volume {}", volumeNumber);
+      return false;
+    }
+    LOG.trace("Cleaning bucket: {} in volume: {}",
+        bucketName, volume.getName());
+    ArrayList<String> keys = new ArrayList<>();
+    try {
+      bucket.listKeys(null).forEachRemaining(x -> keys.add(x.getName()));
+      bucket.deleteKeys(keys);
+      volume.deleteBucket(bucketName);
+      numberOfBucketsCleaned.getAndIncrement();
+      return true;
+    } catch (Throwable e) {
+      exception = e;
+      LOG.error("Could not clean bucket ", e);
+      return false;
+    }
+  }
+
   private OzoneVolume getVolume(Integer volumeNumber) {
     return waitUntilAddedToMap(volumes, volumeNumber);
   }
@@ -1119,4 +1254,9 @@ public final class RandomKeyGenerator implements Callable<Void> {
   public int getThreadPoolSize() {
     return threadPoolSize;
   }
+
+  @VisibleForTesting
+  public void setCleanObjects(boolean cleanObjects) {
+    this.cleanObjects = cleanObjects;
+  }
 }
